#ifndef __COMARRAY_H_INCLUDED__
#define __COMARRAY_H_INCLUDED__

template <class T> class CComArray
{

// Attributes
private:

	int		m_nSize;
	bool		m_bOwner;
	VARIANT* m_pVariant;

// Implemenation
public:

	CComArray() 
		:m_nSize(0),m_bOwner(false),m_pVariant(NULL)
	{ 
	}

	virtual ~CComArray() 
	{ 
		Clear();
	}

	void Clear()
	{
		if(m_bOwner && m_pVariant)
		{
			if(m_pVariant->parray)
				SafeArrayDestroy(m_pVariant->parray);
			m_pVariant->parray=NULL;
			VariantClear(m_pVariant);
		}

		m_nSize=0;
		m_bOwner=false;
		m_pVariant=NULL;
	}

	bool Initialize(VARIANT* pVar,int nSize,bool bOwner)
	{
		if(m_pVariant!=NULL || pVar==NULL || nSize<1)
			return false;

		VariantInit(pVar);
			 
		pVar->vt=VT_ARRAY|VT_UI1;

		SAFEARRAYBOUND s= { nSize*sizeof(T),0 };

		if((pVar->parray=SafeArrayCreate(VT_UI1,1,&s))==NULL)
			return false;
				
		m_nSize=nSize;
		m_bOwner=bOwner;
		m_pVariant=pVar;
		
		return true;	
	}

	bool Attach(VARIANT* pVar,bool bOwner)
	{
		if(m_pVariant!=NULL || pVar==NULL)
			return false;

		if(pVar->parray==NULL || pVar->vt!=(VT_ARRAY|VT_UI1))
			return false;

		long nLBound=0L;
		long nUBound=0L;

		if(FAILED(SafeArrayGetLBound(pVar->parray,1,&nLBound)))
			return false;

		if(FAILED(SafeArrayGetUBound(pVar->parray,1,&nUBound)))
			return false;

		if(nUBound>0L)
			m_nSize=(int)(((nUBound-nLBound)+1L)/sizeof(T));
				
		m_bOwner=bOwner;
		m_pVariant=pVar;
		
		return true;	
	}

	int Size()
	{
		return m_nSize;
	}
	
	bool GetAt(int n,T& Item)
	{
		if(!m_pVariant || n<0 || n>=m_nSize)
			return false;

		LPBYTE pData;
		SafeArrayAccessData(m_pVariant->parray,(void**)&pData);
		CopyMemory(&Item,&pData[n*sizeof(T)],sizeof(T));
		SafeArrayUnaccessData(m_pVariant->parray);
		return true;
	}

	bool SetAt(int n,T& Item)
	{
		if(!m_pVariant || n<0 || n>=m_nSize)
			return false;

		LPBYTE pData;
		SafeArrayAccessData(m_pVariant->parray,(void**)&pData);
		CopyMemory(&pData[n*sizeof(T)],&Item,sizeof(T));
		SafeArrayUnaccessData(m_pVariant->parray);
		return true;
	}
};


#endif __COMARRAY_H_INCLUDED__